/*
 *  Open.Theremin.UNO control software for Arduino.UNO
 *  Version 3.10  alpha
 *  Copyright (C) 2010-2015 by Urs Gaudenz
 *  
 *  Consider: for use with hardware Version 3.0 (with variocap tuning) only ! 
 *
 *  Open.Theremin.UNO control software is free software: you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Open.Theremin.UNO control software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  the Open.Theremin.UNO control software.  If not, see <http://www.gnu.org/licenses/>.
 *   
 *  Volume Switch and Wave table switching by Yasushi Yoshida
 *  Wavetables by Yasushi Yoshida
 *  Version 1.2A - minor 'arduino-izing' - Michael Margolis 20 Oct 2013 
 *  
 *  Extensions for Variocap hardware and software calibration routines and instrument tuning
 *  Expansion of state machine
 *  Adding serial communication for debug use and PC diagnostics and control
 *  by Uwe Kutschki Oct 2015
 *  Test Version
 *  
*/

/*  State machine mode table
 *   
 *   0: wait mode, "warming" and fall-back from calibration
 *   1: hardware calibration mode
 *   2: software calibration mode volume
 *   3: software calibration mode pitch
 *   4: curve param set mode, starting only over serial
 *   5: instrument tuning mode
 *   9: normal play mode
 *   error: error mode
 *
*/ 


/* Serial debug summary; Comment or uncomment the debug lines or blocks for your own intention, but use with care !
 *  
 *  s starts hardware calibration routine (obsolete)
 *  r resets hardware calibration
 *  v read values of beat frequencies
 *  e reads DAC values from EEPROM
 *  i reads ID string from EEPROM
 *  m reads mode state
 *  c switches DAC channel
 *  p adds 16 inc. to DAC value
 *  + adds 1 inc. to DAC value
 *  - substracts 1 inc. to DAC value
 *  a reads analog poti values
 *  t plays test tone sequence
 *  b lets blink - Poti 2 chooses colors
*/ 
  
/* Serial communication command for use with PC service application
 * 
 *  x full status send
 *  y actual vol and pitch 
 *  z actual curve param
 *  0..9 starts mode with equivalent number
 *  
*/

#include "mcpDac.h"
#include <EEPROM.h>

// wavetabel "files" to store in program memory space
#include "theremin_sintable.c"
#include "theremin_sintable2.c"    //adding the extra waveforms
#include "theremin_sintable3.c"
#include "theremin_sintable4.c"
#include "theremin_sintable5.c"
#include "theremin_sintable6.c"
#include "theremin_sintable7.c"
#include "theremin_sintable8.c"

// Version string
const char VerStr[32]="HW V3.1 | SW 3.10 alfa Mar 2016";

// defines of DAC7612 SPI communication
const int S_MOSI = A2;                    // MOSI Data out pin
const int S_SCLK = A3;                    // Clock pin
const int S_LDLA = A4;                    // load latch pin
const int S_CS = A5;                      // chip select pin
const unsigned int adress_v = 0x2000;     // adress DAC channel "volume"
const unsigned int adress_p = 0x3000;     // adress DAC channel "pitch"                

// defines for buttons and debouncing
const int But_pins [3] = {4,12,9} ;       // Array fot button pins hardware Vers. 3.0
const int Num_buts = 3;                   // number of buttons: 3

int Buttons [3] = {0};                    // Variable for button coding: 1 for normal, -1 for long press
boolean but_stat = false;                 // Variable actual button state
boolean old_but_stat [3] = {false};       // Variable old  button state
int but_cnt [3] = {0};                    // counter for button events

// defines for colored LED in "slow mode", pin definition
const int LED_pin_gn = 6;       // green LED pin
const int LED_pin_rd = 5;       // red LED pin
// LED colors
const byte off = 0;
const byte red = 1;
const byte grn = 2;
const byte yel = 3;

const int Bl_c = 30;               // Blink counter half Period: times * 10ms 

// defines for automatic calibration
const unsigned int cal_freq  = 28500;        // Frequency to adjust in calibration mode, makes around 590 Hz
const unsigned int cal_lim   = 2500;         // Limits for calibration set frequency
const unsigned int cal_catch = 1500;         // Limit for catching set frequency while running calibrating routine
const byte error    = 0xFF;                  // mode number for calibration error
const byte cc       = 5;                     // counter for settling cycles of auto calibration
const unsigned int d_max_v = 5000;           // "maximum" value of "vol" in closed hand postion to antenna
const unsigned int d_max_p = 5000;           // "maximum" value of "pitch" in closed hand postion to antenna
const unsigned int Tone_range = 48;          // number of playable tones: 48 means 4 octaves

// defines for EEPROM memory variables to save values to initialise tunings (variocap voltage, max freqencies, curve params)
const unsigned int EE_pit = 0;        // Variocap DAC value volume channel
const unsigned int EE_vol = 2;        // Variocap DAC value volume channel
const unsigned int EE_vmax = 4;       // maximum of value "vol" from software calibration mode
const unsigned int EE_pmax = 6;       // maximum of value "pitch" from software calibration mode
const unsigned int EE_tmin = 8;       // marker value of minimum playable tone
const unsigned int EE_tmax = 10;      // marker value of maximum playable tone
const byte EE_cp = 12;                // marker value for curve parameter in fscale function

#define BAUD 115200 // baud rate for serial comm. for debug use
// defines for speed LED speed control for measuring tasks for debug use
#define ledPin 5                          // LED on D5
#define ledPin1 6                         // LED on D5
#define LED_ON  (PORTD |= (1<<PORTD5))
#define LED_OFF (PORTD &= ~(1<<PORTD5))
#define LED1_ON  (PORTD |= (1<<PORTD6))
#define LED1_OFF (PORTD &= ~(1<<PORTD6))

#define INT0_STATE  (PIND&(1<<PORTD2))    // input pin, signal from volume beat-note mixer
#define PC_STATE  (PINB&(1<<PORTB0))      // input pin, signal from pitch beat-note mixer

// General variables 
int32_t pitch_init       = 0;       // Initialization value of pitch
int32_t vol_init         = 0;       // Initialization value of volume
int32_t pitch_v,pitch_l;            // Last value of pitch (for filtering)
int32_t vol_v,vol_l;                // Last value of volume (for filtering)
uint16_t pitch           = 0;       // Pitch value
uint16_t pitch_counter   = 0;       // Pitch counter
uint16_t pitch_counter_l = 0;       // Last value of pitch counter
uint16_t vol             = 0;       // Volume value
uint16_t vol_counter_l   = 0;       // Last value of volume counter
uint16_t pot1            = 0;       // "volume" potentiometer value 
uint16_t pot2            = 0;       // "wavetable bank" potentiometer value
uint16_t pot_v           = 0;       // Volume value from potentiometer  

uint8_t i;
uint8_t mode      = 0;              // Process mode variable
uint8_t blnk      = 0;              // blink counter
uint8_t act_col   = 0;              // actual LED color
uint8_t alt_col   = 0;              // alternative LED color for blink
uint16_t DAC_val  = 0;              // DAC7612 value
uint8_t Cal_pass  = 0;              // Calibration pass counter
uint8_t c_wait    = 0;              // Calibration wait counter

// variables for calibration
int addr;                           // temp variable for DAC adress
int meas;                           // temp variable for pitch value
int old_meas;                       // marker for control of too big jumps
byte ead;                           // temp variable for EEPROM adress
boolean F_write = false;            // temp flag for writing to EEPROM
uint16_t Max_v = 8000;              // "maximum" value of "vol" in closed hand postion to antenna
uint16_t Max_p = 8000;              // "maximum" value of "pitch" in closed hand postion to antenna
uint16_t Min_tone = 34;             // add_var for minimal playable tone C0
uint16_t Max_tone = 8778;           // maximal playable tone C8
float C_par = 0.0;                  // curve parameter for fscale
// Important Note: the "maximum" values above are smaller than the "xxx_init" values!
// The reason is the measurement of periodes, the higher the frequency the lower is the period...

// Flag variables for process control
boolean F_blnk = false;             // Blink flag
boolean F_but_change = false;       // Flag for button change
boolean F_mode_change = false;      // Flag for mode change
boolean F_cal = false;              // Calibration flag
boolean F_wait = false;             // Flag for alternating of tuning output and measuring frequency
boolean F_hw_cal = false;           // Flag for successful hardware calibration

// volatile variables  - used in the ISR Routine*/
volatile uint8_t addr1        = 0;                // wavetable switch adress
volatile uint8_t vol8;                            // Volume byte
volatile bool flag_vol,flag_pitch           = 0;  // Volume and pitch read flag
volatile uint16_t vol_counter,vol_counter_i = 0;  // Volume counter
volatile uint16_t pointer     = 0;                // Table pointer
volatile uint16_t add_val     = 0;                // Table pointer increment
volatile uint16_t key_note    = 200;              // Value for tuning fundamental note (for later use)
volatile uint16_t timer       = 0;                // Timer value
volatile uint8_t deb_p,deb_v  = 0;                // Counters vor debouncing of high speed timer events

//  debug variables
uint16_t DAC_chan = 1;            // for DAC channel select
uint16_t deb_addr ;               // debug marker for DAC adress

// General Setup Routine 
void setup() {

  // Setup serial communication
  Serial.begin(BAUD);
  
  //setup buttons
  pinMode(But_pins[0],INPUT_PULLUP);
  pinMode(But_pins[1],INPUT_PULLUP);
  pinMode(But_pins[2],INPUT_PULLUP);
  
  // setup DAC7612 pins
  pinMode(S_MOSI, OUTPUT);
  pinMode(S_SCLK, OUTPUT);
  pinMode(S_LDLA, OUTPUT);
  pinMode(S_CS, OUTPUT);
  
  // setup pins of colored LED in "slow mode"
  pinMode(LED_pin_rd, OUTPUT);  
  pinMode(LED_pin_gn, OUTPUT);

  LED_col(red);             // LED to red
  
  outDAC7612(0,0);            // initial value to DAC7612

  /* Setup Timer 1, 16 bit timer used to measure pitch and volume frequency */
  TCCR1A = 0;                         // Set Timer 1 to Normal port operation (Arduino does activate something here ?)
  TCCR1B = (1<<ICES1)|(1<<CS10);      // Input Capture Positive edge select, Run without prescaling (16 Mhz)
  TIMSK1 = (1<<ICIE1);                // Enable Input Capture Inrrupt

  /* Setup interrupts for Wave Generator and Volume read */
  EICRA = (1<<ISC00)|(1<<ISC01)|(1<<ISC11)|(1<<ISC10) ;   // The rising edges of INT0 and INT1 generate an interrupt request.
  EIMSK = (1<<INT0)|(1<<INT1);                            // Enable External Interrupt INT0 and INT1

  interrupts();             // Enable Interrupts (SEI); ...unleashes the Theremin...

 // for debug you may comment out next block
  mcpDacInit();             // Initialize Digital Analog Converter (DAC)

  vol8=127;                 // Set volume to half
  
  add_val = 462;            // Play welcome sounds A3-A4-A5
  ticktimer(10000);
  add_val = 923;
  ticktimer(10000);
  add_val = 1846;
  ticktimer(20000);
  vol8=0;

  // write version string
  if (EEPROM.read(EE_pit) == 0xFF) for (int indx=0 ; indx<32; indx++) EEPROM.write(indx+991,VerStr[indx]);

  // read out EEPROM for predefined values
  DAC_val = ((EEPROM.read(EE_pit)) | (EEPROM.read(EE_pit+1)<<8)); // read initial tuning voltage value for pitch from EEPROM
  if (DAC_val == 65535) DAC_val = 0;                               // if EEPROM is very first set tuning voltage to 0
  outDAC7612(DAC_val,adress_p);                                   // write to DAC
  
  DAC_val = ((EEPROM.read(EE_vol)) | (EEPROM.read(EE_vol+1)<<8)); // read initial tuning voltage value for volume from EEPROM
  if (DAC_val == 65535) DAC_val = 0;                               // if EEPROM is very first set tuning voltage to 0
  outDAC7612(DAC_val,adress_v);                                   // write to DAC

  Max_v = ((EEPROM.read(EE_vmax)) | (EEPROM.read(EE_vmax+1)<<8)); // read initial "maximum" vol value from EEPROM
  if (Max_v == 65535) Max_v = d_max_v;                             // if EEPROM is very first set default value

  Max_p = ((EEPROM.read(EE_pmax)) | (EEPROM.read(EE_pmax+1)<<8)); // read initial "maximum" pitch value from EEPROM
  if (Max_p == 65535) Max_p = d_max_p;                             // if EEPROM is very first set default value

  Min_tone = ((EEPROM.read(EE_tmin)) | (EEPROM.read(EE_tmin+1)<<8)); // read initial highest playable tone from EEPROM
  if (Min_tone == 65535) Min_tone = cal_freq;                         // if EEPROM is very first set default value

  Max_tone = ((EEPROM.read(EE_tmax)) | (EEPROM.read(EE_tmax+1)<<8)); // read initial lowest playable tone from EEPROM
  if (Max_tone == 65535) Max_tone = Max_p;                            // if EEPROM is very first set default value

  byte tmp_c = EEPROM.read(EE_cp);          // read initial marker for curve parameter
  if (tmp_c != 255) C_par = 4.0 * 127 / tmp_c;   // calculate curve parameter       
 
  ticktimer(50000);

  if ((pitch < cal_freq+2*cal_lim) && (pitch > cal_freq-cal_lim)) mode = 0; else mode = 1;  // test set freq. of pitch and select mode
  if (mode == 0) 
    if ((vol < cal_freq+2*cal_lim) && (vol > cal_freq-cal_lim)) mode = 0; else mode = 1;      // test set freq. of volume and select mode

  mode = 0;   // force mode 0 for debug use
  F_mode_change = true;
  
  InitValues();            // Capture initial calibration values

}



/* Main Loop */
void loop() {

  timer = 0; 
  while (timer < 312) {                         // loop for time critical bussiness
    if (flag_pitch){                            // If pitch capture event
      pitch_v=pitch;                            // Averaging pitch values (simple FIR)
      pitch_v=pitch_l+((pitch_v-pitch_l)>>2); 
      pitch_l=pitch_v;    

      add_val=(pitch_init-pitch_v)/2+key_note;  // set pitch increment with fundamental note
      // double tmp_pitch = fscale(pitch_init,Max_p,Min_tone,Max_tone,pitch_init-pitch_v,C_par);
      // add_val= int(tmp_pitch);
      flag_pitch=false;                         // Clear pitch capture flag
      }

    if (flag_vol){                              // If volume capture event
      // if (vol<5000) vol=5000;
      vol_v=vol;                                // Averaging volume values (simple FIR)
      vol_v=vol_l+((vol_v-vol_l)>>2);
      vol_l=vol_v;
                          
      if (mode == 9) {
        vol_v = map(vol_v,Max_v,vol_init,0,4095);  // maps volume to beat frequency limits 
        vol_v = vol_v-(4096-(pot_v<<2));          // calculate volume including potentiometer parameter
        vol_v = constrain(vol_v,0,4096);          // set volume limits      
        vol8=vol_v>>4;                            // set value for calculation in ISR
      }
            
      flag_vol=false;                           // Clear volume capture flag
      }
        
    } ;    // after 10ms do following...

  // LED1_ON;       // debug use
  
  debounce();                 // read buttons

  pot1 = analogRead(0);         // scanning the values of pot 1
  pot2 = analogRead(1);         // scanning the values of pot2
  addr1 = pot2>>7;              // set wavetable adress
  if (mode == 9) pot_v = pot1;

  // addr1=0; // for debug use

// /* Debug routine with serial communication...
  if (Serial.available() > 0){                // uncomment if debug use
     char ch = Serial.read();               // type a single letter to trigger functions
     EvalCh (ch);
  }
// */ 

  /*   debug use block
  if (F_but_change) {
    Serial.println("Button read");
    Serial.println(Buttons[0]);
    Serial.println(Buttons[1]);
    Serial.println(Buttons[2]);
    // F_but_change = false;
    }
  */ 
  
  // if (mode == 9) Serial.println(vol8); 

// /*  debug use block 
  if (F_but_change) {           // set mode if button changed
    if (Buttons[0] == 1) {      // main button short press
      switch (mode) {
      case 0: mode = 9;         // set normal play mode
        break;
      case 9: mode = 0;         // or set in "wait mode" for pausing the play and hold it warm...
        break;
      case 4: save_cur();       // save curve parameter marker
        break;
      case 5: save_tones();     // from tuning save the max tones
      }
      }
    if (Buttons[0] == -1) {     // main button long press
      mode = 1;                 // set calibration mode
      }
    if (Buttons[1] == 1) {
      if (mode == 2) save_vmax(); // save "maximum" vol value
      }     
    if (Buttons[1] == -1) {
      if (mode == 0) mode = 2;  // set software calibration volume
      }
    if (Buttons[2] == 1) {
      switch(mode) {
      case 0: mode = 5;         // set instrument tuning
        break;
      case 3: save_pmax();  // save "maximum" pitch value
      }
      }     
    if (Buttons[2] == -1) {
      if (mode == 0) mode = 3;  // set software calibration pitch
      }   
    F_but_change = false;       // reset flag for button change
    F_mode_change = true;       // set flag for mode change
    }
// */    

  // F_mode_change = false;       // debug use
  
  if(F_mode_change) {
    switch (mode){              // set state machine when mode
      case 0: {                 // set wait mode: waits for button press to change to another mode, good to warm up
        pot_v = 0;
        vol8 = 0;
        Blink (grn,off);
        } break;
      case 1: {                 // set calibration mode
        pot_v = 0;
        Blink(yel,off);
        F_cal = true;
        initCal();
        } break;
      case 2: {                 // set software calibration for volume
        Blink (grn,yel);
        Max_v = 0;
        Max_v = vol_init;
        proc_vmax();
        } break;
      case 3: {                 // set software calibration for
        Blink (red,yel);
        Max_p = 0;
        Max_p = pitch_init;
        proc_pmax();
        } break;
      case 4: {                 // set curve parameter for use in fscale (see there)
        Blink (red,off);
        cur_set();
        } break;
      case 5: {                 // starts instrument tuning routine
        Blink (off,off);
        LED_col(yel);
        vol8 = 200;
        tuning();
        } break;
      case 9: {                   // set normal play mode
        Blink(off,off);
        LED_col(grn);
        InitValues();             // Capture initial calibration values
        } break;
      case error: {               // set error mode, results in inifinite loop !
        DAC_val = 0;
        outDAC7612 (DAC_val,0);
        Blink (red,off);
        } break;
      }
    F_mode_change = false;
    play_sign();
  }

  if (mode == 2) proc_vmax();
  if (mode == 3) proc_pmax();
    
  if (F_blnk) {                       // slow blink routine
    blnk++;
    if (blnk == Bl_c) LED_col(alt_col);   // turn LED off after first period or alterantive color
    if (blnk == (2*Bl_c)) {           // after second period..
      LED_col(act_col);               // turn LED on
      blnk = 0;                       // ..and reset counter
      }       
    }  

  // F_cal = false;       // debug use
  
  if (F_cal) {                                          // Calibration routine for variocap tuning, uses 2 passes
    c_wait ++;
    if (c_wait == cc) {                                 // waits for 'cc' cycles to settle DAC values and frequencies
      switch (Cal_pass) {
       case 1: {                                         // pass 1 calibrates volume
         addr = adress_v;                                // set DAC adress
         meas = vol_v;                       // set measured value
         if (meas<1000) meas = 1000;
         } break;
       case 2: {                                         // pass 2 calibrates pitch
         addr = adress_p;                                // set DAC adress
         meas = pitch_v;                       // set measured value
         if (meas<1000) meas = 1000;                       
         } break;
      }
      if ((meas-old_meas) > 5000) meas = old_meas;
      else old_meas = meas;
      if (F_wait){                                        // makes frequency test in next loop
        if (meas < (cal_freq-5*cal_catch)) DAC_val += 32;
          else DAC_val ++;                                // calculate tuning voltage
        if (DAC_val < 0x0FFF) outDAC7612(DAC_val,addr);   // write to DAC 
          else {
           mode = error;                                 // if DAC overflow raise error
            F_mode_change = true;
            F_cal = false;
            }                    
          }
        else if (meas > (cal_freq - cal_catch)) F_write = true;   // if pitch matches beat-note set flag to save DAC value
      F_wait = ! F_wait;
      if (F_write) {
        if (Cal_pass == 1) ead = EE_vol; else ead = EE_pit;
        byte vp lowByte(DAC_val);
        EEPROM.write(ead,vp);
        vp = highByte(DAC_val);
        EEPROM.write(ead+1,vp);
        if (Cal_pass == 1) vol_init = meas;
        if (Cal_pass == 2) pitch_init = meas;
        Cal_pass ++;
        DAC_val=0;
        F_write = false;
        }
      if (Cal_pass > 2){
        F_cal = false;
        mode = 0;
        F_mode_change = true;
        F_hw_cal = true;
        }
      c_wait = 0;
    }
  }    

  // LED1_OFF;       // debug use

}

/* Here under the ISRs and 

/* 16 bit by 8 bit multiplication */
static inline uint32_t mul_16_8(uint16_t a, uint8_t b) {
  uint32_t product;
  asm (
    "mul %A1, %2\n\t"
    "movw %A0, r0\n\t"
    "clr %C0\n\t"
    "clr %D0\n\t"
    "mul %B1, %2\n\t"
    "add %B0, r0\n\t"
    "adc %C0, r1\n\t"
    "clr r1"
: 
    "=&r" (product)
: 
    "r" (a), "r" (b));
  return product;
  }

/* Externaly generated 31250 Hz Interrupt for WAVE generator (32us) */
ISR (INT1_vect) {
  // Interrupt takes up a total of max 25 us
  EIMSK &= ~ (1<<INT1);     // Disable External Interrupt INT1 to avoid recursive interrupts
  interrupts();             // Enable Interrupts to allow counter 1 interrupts

  int16_t temp_val;         // temporary variable 1
  uint32_t temp2_val;       // temporary variable 2

    switch (addr1)					// set wave tables for reading
  	{
    case 0:  temp_val = (signed int)pgm_read_word_near (sine_table + ((unsigned int)(pointer>>6) & 0x3ff)); break;  //Read next wave table value (3.0us)
    case 1:  temp_val = (signed int)pgm_read_word_near (sine_table2 + ((unsigned int)(pointer>>6) & 0x3ff)); break; //Read next wave table value (3.0us)
    case 2:  temp_val = (signed int)pgm_read_word_near (sine_table3 + ((unsigned int)(pointer>>6) & 0x3ff)); break; //Read next wave table value (3.0us)
    case 3:  temp_val = (signed int)pgm_read_word_near (sine_table4 + ((unsigned int)(pointer>>6) & 0x3ff)); break; //Read next wave table value (3.0us)
    case 4:  temp_val = (signed int)pgm_read_word_near (sine_table5 + ((unsigned int)(pointer>>6) & 0x3ff)); break;  //Read next wave table value (3.0us)
    case 5:  temp_val = (signed int)pgm_read_word_near (sine_table6 + ((unsigned int)(pointer>>6) & 0x3ff)); break; //Read next wave table value (3.0us)
    case 6:  temp_val = (signed int)pgm_read_word_near (sine_table7 + ((unsigned int)(pointer>>6) & 0x3ff)); break; //Read next wave table value (3.0us)
    case 7:  temp_val = (signed int)pgm_read_word_near (sine_table8 + ((unsigned int)(pointer>>6) & 0x3ff)); break; //Read next wave table value (3.0us)
    };

  //  LED_ON;
  if (temp_val>0){                      // multiply 16 bit wave number by 8 bit volume value (11.2us / 5.4us)
    temp2_val=mul_16_8(temp_val,vol8);
    temp2_val=temp2_val>>9; 
    temp2_val=temp2_val+1748;  
  } 
  else
  { 
    temp2_val=mul_16_8(-temp_val,vol8);
    temp2_val=temp2_val>>9; 
    temp2_val=1748-temp2_val;
  }

  //  LED_OFF;
  mcpDacSend(temp2_val);          //Send result to Digital to Analogue Converter (audio out) (9.6 us)

  pointer += add_val;             // increment table pointer (ca. 2us)
  timer++;                        // update 32us timer

  if (PC_STATE) deb_p++;          // pitch pin counts debouncer
  
  if (deb_p==3) {                            // calculate pitch after 3 bounces
    cli();
    pitch_counter=ICR1;                      // Get Timer-Counter 1 value
    pitch=(pitch_counter-pitch_counter_l);   // Counter change since last interrupt -> pitch value
    pitch_counter_l=pitch_counter;           // Set actual value as new last value
  };
  
  if (deb_p==5) flag_pitch=true;            // set flag after 5 bounces
    
  if (INT0_STATE) deb_v++;                  // pitch pin counts debouncer
   
  if (deb_v==3) {
    cli();
      vol_counter=vol_counter_i;            // Get Timer-Counter 1 value
      vol=(vol_counter-vol_counter_l);      // Counter change since last interrupt 
      vol_counter_l=vol_counter;            // Set actual value as new last value
  };
    
  if (deb_v==5) flag_vol=true;              // set flag after 5 bounces

  cli();                        // Turn of interrupts
  EIMSK |= (1<<INT1);           // Re-Enable External Interrupt INT1

}

/* VOLUME read - interrupt service routine for capturing volume counter value */
ISR (INT0_vect) {   
  vol_counter_i = TCNT1;
  deb_v=0;
}


/* PITCH read - interrupt service routine for capturing pitch counter value */
ISR (TIMER1_CAPT_vect) {
  deb_p=0;
}


void ticktimer (int ticks) {     // Wait for ticks * 32 us
  timer=0;
  while(timer<ticks);
}


void InitValues(void) {                     // Initialize pitch and volume values for calibration, not used after hardware calibration
  if (F_hw_cal == false) { 
    // Set initial pitch value
    pitch_counter_l=pitch_counter;            // Store actual Timer 1 counter value
    flag_pitch=false;                         // Clear pitch flag
    timer=0;
    while(!flag_pitch && (timer < 312));      // Wait for new value (with exit after 10 ms)
    pitch_init=pitch;                         // Counter change since last interrupt = init pitch value
    // Set initial volume value   
    vol_counter_l=vol_counter;                // Store actual counter value
    flag_vol=false;                           // Clear volume flag
    timer=0;
    while(!flag_vol && (timer < 312));        // Wait for new value (with exit after 10 ms)
    vol_init=vol;                             // Counter change since last interrupt = init volume value
  }
}


void outDAC7612 (unsigned int out_val, unsigned int adr) {  // Subroutine for DAC7612 output: 12bit value and 2bit adress (coded in define)
  unsigned char i;
  unsigned int output;
  digitalWrite(S_CS,0);                 // set chip select low
  output = (out_val | adr) << 2;        // make 14 bit output value
  for (i = 14; i; i--) {                // 14 times loop
  digitalWrite(S_SCLK,0);               // write clock falling edge
  if (output & 0x8000) {                // write data bit
     digitalWrite(S_MOSI,1);}           // 1 if data bit i is high
     else {
     digitalWrite(S_MOSI,0);}           // 0 if data bit i is low
  output = output << 1;                 // shift data word
  digitalWrite(S_SCLK,1);               // write clock raising edge
  }                                     // loop end
  digitalWrite(S_CS,1);                 // set chip selct high
  digitalWrite(S_LDLA,0);               // lath data with falling edge of LDLA
  digitalWrite(S_LDLA,1);               // raise LDLA
  digitalWrite(S_MOSI,1);               // raise data pin
}


void debounce () {                                // Subroutine for debouncing buttons low-active mit pullup, to execute every 10ms! 
  F_but_change = false;                         // clean Flag for button change
  for ( int i = 0; i < Num_buts; i++){            // loop for every button
    int but_read = digitalRead ( But_pins[i] );   // read condition of button i
    if (but_read == LOW ) but_stat = true;        // copy to status
      else but_stat = false;
    if (but_stat) {                               // if button pressed... 
      if (but_stat == old_but_stat [i]) {         // compare if button is still pressed ..
        but_cnt [i] ++;                           // increment counter
        if (but_cnt [i] > 80) {                   // longer than 800 ms pressed ?
          Buttons [i] = 0;                        // reset coding
          Buttons [i] = -1;                       // code button as "Long press"             
          but_cnt [i] = 0;                        // reset counter
          F_but_change = true;                    // set Flag: button has changed
          }
        }         
      } else {                                    // if button not (..no longer) pressed...
        if (but_stat == old_but_stat [i]) {       // compare if button is still loose ..
          Buttons [i] = 0;                        // reset code        
          int erg = constrain ( but_cnt[i], 3, 40); // is counter >3 and <40
          if (erg == but_cnt[i]) {                // if counter is between 3 and 40
            Buttons [i] = 1;                      // code button as "Normal press"
            } 
            else if (erg == 40) {                 // if counter > 40
              Buttons [i] = -1;                   // code button as "Long press"
              } 
          but_cnt [i] = 0;                        // reset counter
          if (erg != 3) F_but_change = true;      // set Flag: button has changed 
        }
      }  
    old_but_stat [i] = but_stat;                  // set old status as actual status
  }
}

void LED_col (byte color) {                         // Set routine for color LED
  if (color & 0x01) digitalWrite(LED_pin_rd , 1);
    else digitalWrite(LED_pin_rd , 0); 
  if (color & 0x02) digitalWrite(LED_pin_gn , 1);
    else digitalWrite(LED_pin_gn , 0); 
}

void Blink ( byte c1, byte c2) {
  if (c1 == off) {
    LED_col (act_col);
    F_blnk = false;
    } else {
      act_col = c1;
      alt_col = c2;
      F_blnk = true;
      }
  }

void initCal () {
  uint8_t tvol = pot1>>4;
  outDAC7612(0,0);          // initial value to DAC7612
  Cal_pass = 1;             // set pass counter
  vol8=tvol;                 // Play calibration sounds
  add_val = 490;
  ticktimer(15000);
  vol8=0;
  ticktimer(3000);
  vol8=tvol;
  ticktimer(15000);
  vol8=0;
  ticktimer(3000);
  vol8=tvol;   
  ticktimer(15000);
  vol8=0;
  ticktimer(3000);
  add_val = 2765;
  vol8=tvol;
  ticktimer(3000);
  vol8=0;
  DAC_val = 0;            // set initial Value for tuning voltage
  outDAC7612(DAC_val,0);
  F_wait = false;         // set flag for alternating of tuning output and measuring frequency
  c_wait = 0;
  Cal_pass = 1;
  old_meas = 1000;
}

void play_sign () {
  for (i=0;i<mode;i++) {
    uint8_t tvol = pot1>>2;
      vol8=tvol;
      add_val = 923;   
      ticktimer(5000);
      vol8=0;
      ticktimer(5000);
      vol8=tvol;
      add_val = 0;  
    }
}

void proc_vmax () {       // procedure determines "maximum" value of "vol" when hand ist in closest position to antenna 
  if (vol_v < Max_v) {
    Max_v = vol_v;
    Serial.println(Max_v);
  }
  }

void save_vmax () {       // saves "maximum" value of "vol" to EEPROM
  byte tmp_v = lowByte(Max_v);
  EEPROM.write(EE_vmax,tmp_v);
  tmp_v = highByte(Max_v);
  EEPROM.write(EE_vmax+1,tmp_v);
  mode = 0;
  F_mode_change = true;
  }  

void proc_pmax () {       // procedure determines "maximum" value of "pitch" when hand ist in closest position to antenna
  if (pitch_v < Max_p) Max_p = pitch_v;
  }

void save_pmax () {       // saves "maximum" value of "pitch" to EEPROM
  byte tmp_v = lowByte(Max_p);
  EEPROM.write(EE_pmax,tmp_v);
  tmp_v = highByte(Max_p);
  EEPROM.write(EE_pmax+1,tmp_v);
  mode = 0;
  F_mode_change = true;
  } 

void save_tones() {       // saves the min and max tones after tuning
  byte tmp_v = lowByte(Min_tone);
  EEPROM.write(EE_tmin,tmp_v);
  tmp_v = highByte(Min_tone);
  EEPROM.write(EE_tmin+1,tmp_v);
  tmp_v = lowByte(Max_tone);
  EEPROM.write(EE_tmax,tmp_v);
  tmp_v = highByte(Max_tone);
  EEPROM.write(EE_tmax+1,tmp_v);
  mode = 0;
  F_mode_change = true;  
  } 

void cur_set() {          // sets curve parameter
  //
  }

void save_cur() {         // save curve parameter marker
  //
  mode = 0;
  F_mode_change = true;  
  }

void tuning () {          // procedure for instrument tuning
  //
  }

void EvalCh (char start) {                    // evaluate serial command, extra routine for better servicing
      switch (start) {
      case 's': {                             // s starts calibration routine
        Serial.println("Start cal");
        Serial.println(pitch_v);
        Serial.println(vol_v);
        mode = 1;
        F_mode_change = true;
        }      
      break;
      case 'r': {                             // r resets calibration
        Serial.println("Reset cal");
        mode = 0;
        F_mode_change = true;
        F_cal = false;
        DAC_val = 0;
        outDAC7612 (DAC_val,0);
        }
        break;
      case 'v': {                              // v read values of beat frequencies
        Serial.println("read pitch & vol");
        Serial.println(pitch);
        Serial.println(vol);
        Serial.println("DAC value");
        Serial.println(DAC_val);
        }
      break;
      case 'e':{                                // e reads DAC values from EEPROM
        Serial.println("EEPROM read");
        int val = ((EEPROM.read(EE_pit)) | (EEPROM.read(EE_pit+1)<<8));        
        Serial.println(val);
        val = ((EEPROM.read(EE_vol)) | (EEPROM.read(EE_vol+1)<<8));        
        Serial.println(val);
        }
        break;
      case 'i':{                                // i reads ID string from EEPROM
        Serial.println("OTU ID read");
        char txt_id[32];
        for (i=0;i<32;i++) txt_id[i] = EEPROM.read(959+i);       
        Serial.println(txt_id);
        for (i=0;i<32;i++) txt_id[i] = EEPROM.read(991+i);       
        Serial.println(txt_id);
        }
        break;
      case 'm':{                                // m reads mode state
        Serial.println("Mode");
        Serial.println(mode);
        F_but_change = false;
        }
        break;
      case 'c':{                                // c switches DAC channel
        if ( DAC_chan == 0) {
          DAC_chan = 1;
          deb_addr = adress_p;
          Serial.println("pitch channel");
        } else {
          DAC_chan = 0;
          deb_addr = adress_v;
          Serial.println("volume channel");
          }     
        }
        break;
      case 'p':{                                // p adds 16 inc. to DAC value
        Serial.println("DAC +16");
        DAC_val += 16;
        outDAC7612 (DAC_val,deb_addr);
        Serial.println(DAC_val);
        }
        break;
      case '+':{                                // + adds 1 inc. to DAC value
        Serial.println("DAC +1");
        DAC_val ++;
        outDAC7612 (DAC_val,deb_addr);
        Serial.println(DAC_val);
        }
        break;
      case '-':{                                // - substracts 1 inc. to DAC value
        Serial.println("DAC -1");
        DAC_val --;
        outDAC7612 (DAC_val,deb_addr);
        Serial.println(DAC_val);
        }
        break;
      case 'a':{                                // a reads analog poti values
        Serial.println("Poti read");
        Serial.println(pot1);
        Serial.println(pot2);
        Serial.println(addr1);
        }
        break;
      case 't':{                                // t plays test tone sequence
        Serial.println("Tone sequence");
        uint8_t tmp_mode = mode;
        mode = 6;
        play_sign();
        mode = tmp_mode;
        }
        break;
      case 'b':{                                // b lets blink - Poti 2 chooses colors
        Serial.println("Blink");
        byte f1, f2;
        switch (addr1){
          case 0: { f1 = red; f2 = off; }
            break;
          case 1: { f1 = grn; f2 = off; }
            break;
          case 2: { f1 = yel; f2 = off; }
            break;
          case 3: { f1 = grn; f2 = red; }
            break;
          case 4: { f1 = grn; f2 = yel; }
            break;
          case 5: { f1 = yel; f2 = red; }
            break;
          case 6: { f1 = yel; f2 = red; }
            break;
          case 7: { f1 = off; f2 = off; }
            break;
          };
        Blink ( f1, f2);
        }
        break;
      case 'x':{                                // full status red
        Serial.println(mode);
        int val = ((EEPROM.read(EE_pit)) | (EEPROM.read(EE_pit+1)<<8));        
        Serial.println(val);
        val = ((EEPROM.read(EE_vol)) | (EEPROM.read(EE_vol+1)<<8));        
        Serial.println(val);
        val = ((EEPROM.read(EE_vmax)) | (EEPROM.read(EE_vmax+1)<<8));        
        Serial.println(val);
        val = ((EEPROM.read(EE_pmax)) | (EEPROM.read(EE_pmax+1)<<8));        
        Serial.println(val);
        val = ((EEPROM.read(EE_tmin)) | (EEPROM.read(EE_tmin+1)<<8));        
        Serial.println(val);
        val = ((EEPROM.read(EE_tmax)) | (EEPROM.read(EE_tmax+1)<<8));        
        Serial.println(val);
        val = EEPROM.read(EE_cp);
        Serial.println(val);        
        Serial.println(vol_init);        
        Serial.println(pitch_init);        
        Serial.println(Max_v);
        Serial.println(Max_p);
        Serial.println(Min_tone);        
        Serial.println(Max_tone);
        Serial.println(C_par);
        Serial.println(vol_v);        
        Serial.println(pitch_v);
        Serial.println(add_val);      
        //
        }
        break;
      case 'y':{                                // actual vol and pitch
        Serial.println(vol_v);
        Serial.println(pitch_v);        
        }
        break;
      case 'z':{                                // actual curve param
        Serial.println(C_par);
        }
        break;
      case '0': {                             // 0 starts mode 0
        Serial.println("Mode 0");
        mode = 0;
        F_mode_change = true;
        }      
        break;
      case '1': {                             // 1 starts mode 1
        Serial.println("Mode 0");
        mode = 1;
        F_mode_change = true;
        }      
        break;
      case '2': {                             // 2 starts mode 2
        Serial.println("Mode 0");
        mode = 2;
        F_mode_change = true;
        }      
        break;
      case '3': {                             // 3 starts mode 3
        Serial.println("Mode 0");
        mode = 3;
        F_mode_change = true;
        }      
        break;
      case '4': {                             // 4 starts mode 4
        Serial.println("Mode 0");
        mode = 4;
        F_mode_change = true;
        }      
        break;
      case '5': {                             // 5 starts mode 5
        Serial.println("Mode 0");
        mode = 5;
        F_mode_change = true;
        }      
        break;
      case '9': {                             // 9 starts mode 9
        Serial.println("Mode 0");
        mode = 9;
        F_mode_change = true;
        }      
        break;
      }
  }

/*  Function for scaling of values from an original range to a mapping range
 *   uses a "curve" parameter for bending linear curve in logarithmic manner
 *  For further information read http://playground.arduino.cc/Main/Fscalehttp://playground.arduino.cc/Main/Fscale
 */
 
float fscale( float originalMin, float originalMax, float newBegin, float
newEnd, float inputValue, float curve){

  float OriginalRange = 0;
  float NewRange = 0;
  float zeroRefCurVal = 0;
  float normalizedCurVal = 0;
  float rangedValue = 0;
  boolean invFlag = 0;

  // condition curve parameter
  // limit range
  if (curve > 10) curve = 10;
  if (curve < -10) curve = -10;

  curve = (curve * -.1) ; // - invert and scale - this seems more intuitive - postive numbers give more weight to high end on output 
  curve = pow(10, curve); // convert linear scale into lograthimic exponent for other pow function
  /* debug...
   Serial.println(curve * 100, DEC);   // multply by 100 to preserve resolution  
   Serial.println(); 
   */
  // Check for out of range inputValues
  if (inputValue < originalMin) {
    inputValue = originalMin;
  }
  if (inputValue > originalMax) {
    inputValue = originalMax;
  }
  // Zero Refference the values
  OriginalRange = originalMax - originalMin;
  if (newEnd > newBegin){ 
    NewRange = newEnd - newBegin;
  }
  else
  {
    NewRange = newBegin - newEnd; 
    invFlag = 1;
  }
  zeroRefCurVal = inputValue - originalMin;
  normalizedCurVal  =  zeroRefCurVal / OriginalRange;   // normalize to 0 - 1 float
  /* debug..
  Serial.print(OriginalRange, DEC);  
   Serial.print("   ");  
   Serial.print(NewRange, DEC);  
   Serial.print("   ");  
   Serial.println(zeroRefCurVal, DEC);  
   Serial.println();  
   */
  // Check for originalMin > originalMax  - the math for all other cases i.e. negative numbers seems to work out fine 
  if (originalMin > originalMax ) {
    return 0;
  }
  if (invFlag == 0){
    rangedValue =  (pow(normalizedCurVal, curve) * NewRange) + newBegin;
  }
  else     // invert the ranges
  {   
    rangedValue =  newBegin - (pow(normalizedCurVal, curve) * NewRange); 
  }
  return rangedValue;
}
   

